@@ -78,6 +78,14 @@
 /*
  * Jakarta (jk_) include files
  */
+#ifdef NETWARE
+#define _SYS_TYPES_H_
+#define _NETDB_H_INCLUDED
+#define _IN_
+#define _INET_
+#define _SYS_TIMEVAL_H_
+#define _SYS_SOCKET_H_
+#endif
 #include "jk_global.h"
 #include "jk_util.h"
 #include "jk_map.h"
@@ -91,6 +99,12 @@
 #define JK_MAGIC_TYPE       ("application/x-jakarta-servlet")
 #define NULL_FOR_EMPTY(x)   ((x && !strlen(x)) ? NULL : x) 
 
+/*
+ * If you are not using SSL, comment out the following
+ * line. It will make apache run faster.
+ */
+#define ADD_SSL_INFO    
+
 module MODULE_VAR_EXPORT jk_module;
 
 typedef struct {
@@ -103,7 +117,21 @@ typedef struct {
     jk_map_t *uri_to_context;
     jk_uri_worker_map_t *uw_map;
 
-    int was_initialized;
+    /*
+     * SSL Support
+     */
+    int  ssl_enable;
+    char *https_indicator;
+    char *certs_indicator;
+    char *cipher_indicator;
+    char *sesion_indicator;
+
+    /*
+     * Environment variables support
+     */
+    int envvars_in_use;
+    table *envvars;       
+
     server_rec *s;
 } jk_server_conf_t;
 
@@ -220,27 +248,36 @@ static int JK_METHOD ws_write(jk_ws_service_t *s,
 
         if(l) {
             BUFF *bf = p->r->connection->client;
-            ap_size_t w = (ap_size_t)l;
-            ap_size_t r = 0;
+            char *buf = (char *)b;
+            int w = (int)l;
+            int r = 0;
 
             if(!p->response_started) {
                 if(!s->start_response(s, 200, NULL, NULL, NULL, 0)) {
                     return JK_FALSE;
                 }
             }
-            
-	        
-            ap_bwrite(bf, (const char *)b, w, &r);
-            if(w != r) {
-			    return JK_FALSE;
-            }
 
+            while(l && !p->r->connection->aborted) {
+                w = ap_bwrite(p->r->connection->client, &buf[r], l);
+                if (w > 0) {
+                    ap_reset_timeout(p->r); /* reset timeout after successful write */
+                    r += w;
+                    l -= w;
+                } else if (w < 0) {
+                    if(!p->r->connection->aborted) {
+                        ap_bsetflag(p->r->connection->client, B_EOUT, 1);
+                        p->r->connection->aborted = 1;
+                    }
+                    return JK_FALSE;
+                }
+                
+            }
+           
             /*
              * To allow server push.
              */
-            if(ap_bflush(bf) != APR_SUCCESS) {
-                return JK_FALSE;
-            }
+            ap_bflush(bf);
         }
 
         return JK_TRUE;
@@ -257,17 +294,18 @@ static int JK_METHOD ws_write(jk_ws_service_t *s,
 static void jk_error_exit(const char *file, 
                           int line, 
                           int level, 
-                          server_rec *s,
+                          const server_rec *s,
+                          ap_pool *p,
                           const char *fmt, ...) 
 {
     va_list ap;
     char *res;
 
     va_start(ap, fmt);
-    res = ap_pvsprintf(s->process->pool, fmt, ap);
+    res = ap_pvsprintf(p, fmt, ap);
     va_end(ap);
 
-    ap_log_error(file, line, level, 0, s, res);
+    ap_log_error(file, line, level, s, res);
 
     /* Exit process */
     exit(1);
@@ -292,16 +330,18 @@ static int get_content_length(request_rec *r)
 }
 
 static int init_ws_service(apache_private_data_t *private_data,
-                           jk_ws_service_t *s)
+                           jk_ws_service_t *s,
+                           jk_server_conf_t *conf)
 {
     request_rec *r      = private_data->r;
+    char *ssl_temp      = NULL;
     s->jvm_route        = NULL;
     s->start_response   = ws_start_response;
     s->read             = ws_read;
     s->write            = ws_write;
 
-    s->auth_type    = NULL_FOR_EMPTY(r->ap_auth_type);
-    s->remote_user  = NULL_FOR_EMPTY(r->user);
+    s->auth_type    = NULL_FOR_EMPTY(r->connection->ap_auth_type);
+    s->remote_user  = NULL_FOR_EMPTY(r->connection->user);
 
     s->protocol     = r->protocol;
     s->remote_host  = (char *)ap_get_remote_host(r->connection,
@@ -311,29 +351,77 @@ static int init_ws_service(apache_private_data_t *private_data,
     s->remote_host  = NULL_FOR_EMPTY(s->remote_host);
 
     s->remote_addr  = NULL_FOR_EMPTY(r->connection->remote_ip);
-    s->server_name  = (r->hostname ? r->server->server_hostname : r->hostname);
+    s->server_name  = (char *)(r->hostname ? r->hostname : r->server->server_hostname);
     s->server_port  = r->server->port;
-    s->server_software = ap_get_server_version();
+    s->server_software = (char *)ap_get_server_version();
 
     s->method       = (char *)r->method;
     s->content_length = get_content_length(r);
     s->query_string = r->args;
-    s->req_uri      = r->uri;
-    
+    s->req_uri      = r->unparsed_uri;
+    if (s->req_uri != NULL) {
+	char *query_str = strchr(s->req_uri, '?');
+	if (query_str != NULL) {
+	    *query_str = 0;
+	}
+    }
+
     s->is_ssl       = JK_FALSE;
     s->ssl_cert     = NULL;
     s->ssl_cert_len = 0;
     s->ssl_cipher   = NULL;
     s->ssl_session  = NULL;
 
+    if(conf->ssl_enable || conf->envvars_in_use) {
+        ap_add_common_vars(r);
+
+        if(conf->ssl_enable) {
+            ssl_temp = (char *)ap_table_get(r->subprocess_env, 
+                                            conf->https_indicator);
+            if(ssl_temp && !strcasecmp(ssl_temp, "on")) {
+                s->is_ssl       = JK_TRUE;
+                s->ssl_cert     = (char *)ap_table_get(r->subprocess_env, 
+                                                       conf->certs_indicator);
+                if(s->ssl_cert) {
+    	            s->ssl_cert_len = strlen(s->ssl_cert);
+                }
+                s->ssl_cipher   = (char *)ap_table_get(r->subprocess_env, 
+                                                       conf->cipher_indicator);
+                s->ssl_session  = (char *)ap_table_get(r->subprocess_env, 
+                                                       conf->sesion_indicator);
+            }
+        }
+
+        if(conf->envvars_in_use) {
+            array_header *t = ap_table_elts(conf->envvars);        
+            if(t && t->nelts) {
+                int i;
+                table_entry *elts = (table_entry *)t->elts;
+                s->attributes_names = ap_palloc(r->pool, sizeof(char *) * t->nelts);
+                s->attributes_values = ap_palloc(r->pool, sizeof(char *) * t->nelts);
+
+                for(i = 0 ; i < t->nelts ; i++) {
+                    s->attributes_names[i] = elts[i].key;
+                    s->attributes_values[i] = (char *)ap_table_get(r->subprocess_env, 
+                                                                   elts[i].key);
+                    if(!s->attributes_values[i]) {
+                        s->attributes_values[i] = elts[i].val;
+                    }
+                }
+
+                s->num_attributes = t->nelts;
+            }
+        }
+    }
+
     s->headers_names    = NULL;
     s->headers_values   = NULL;
     s->num_headers      = 0;
     if(r->headers_in && ap_table_elts(r->headers_in)) {
-        ap_array_header_t *t = ap_table_elts(r->headers_in);        
+        array_header *t = ap_table_elts(r->headers_in);        
         if(t && t->nelts) {
             int i;
-            ap_table_entry_t *elts = (ap_table_entry_t *)t->elts;
+            table_entry *elts = (table_entry *)t->elts;
             s->num_headers = t->nelts;
             s->headers_names  = ap_palloc(r->pool, sizeof(char *) * t->nelts);
             s->headers_values = ap_palloc(r->pool, sizeof(char *) * t->nelts);
@@ -384,7 +472,7 @@ static const char *jk_mount_context(cmd_parms *cmd,
      * Add the new worker to the alias map.
      */
     char *old;
-    map_put(conf->uri_to_context, context, worker, &old);
+    map_put(conf->uri_to_context, context, worker, (void **)&old);
     return NULL;
 }
 
@@ -414,6 +502,73 @@ static const char *jk_set_log_file(cmd_parms *cmd,
     return NULL;
 }
 
+static const char *jk_set_enable_ssl(cmd_parms *cmd, 
+                                     void *dummy, 
+                                     int flag) 
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
+    
+    /* Set up our value */
+    conf->ssl_enable = flag ? JK_TRUE : JK_FALSE;
+
+    return NULL;
+}
+
+static const char *jk_set_https_indicator(cmd_parms *cmd, 
+                                          void *dummy, 
+                                          char *indicator)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
+
+    conf->https_indicator = indicator;
+
+    return NULL;
+}
+
+static const char *jk_set_certs_indicator(cmd_parms *cmd, 
+                                          void *dummy, 
+                                          char *indicator)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
+
+    conf->certs_indicator = indicator;
+
+    return NULL;
+}
+
+static const char *jk_set_cipher_indicator(cmd_parms *cmd, 
+                                           void *dummy, 
+                                           char *indicator)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
+
+    conf->cipher_indicator = indicator;
+
+    return NULL;
+}
+
+static const char *jk_set_sesion_indicator(cmd_parms *cmd, 
+                                           void *dummy, 
+                                           char *indicator)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
+
+    conf->sesion_indicator = indicator;
+
+    return NULL;
+}
+
+
 static const char *jk_set_log_level(cmd_parms *cmd, 
                                     void *dummy, 
                                     char *log_level)
@@ -427,37 +582,91 @@ static const char *jk_set_log_level(cmd_parms *cmd,
     return NULL;
 }
 
+
+static const char *jk_add_env_var(cmd_parms *cmd, 
+                                  void *dummy, 
+                                  char *env_name,
+                                  char *default_value)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
+
+    conf->envvars_in_use = JK_TRUE;
+    
+    ap_table_add(conf->envvars, env_name, default_value);
+
+    return NULL;
+}
+
+
 static const command_rec jk_cmds[] =
 {
+    /*
+     * JkWorkersFile specifies a full path to the location of the worker 
+     * properties file.
+     *
+     * This file defines the different workers used by apache to redirect 
+     * servlet requests.
+     */
     {"JkWorkersFile", jk_set_wroker_file, NULL, RSRC_CONF, TAKE1,
      "the name of a worker file for the Jakarta servlet containers"},
+    /*
+     * JkMount mounts a url prefix to a worker (the worker need to be
+     * defined in the worker properties file.
+     */
     {"JkMount", jk_mount_context, NULL, RSRC_CONF, TAKE23,
      "A mount point from a context to a Tomcat worker"},
+    /*
+     * JkMountCopy specifies if mod_jk should copy the mount points
+     * from the main server to the virtual servers.
+     */
     {"JkMountCopy", jk_set_mountcopy, NULL, RSRC_CONF, FLAG,
      "Should the base server mounts be copied to the virtual server"},
+    /*
+     * JkLogFile & JkLogLevel specifies to where should the plugin log
+     * its information and how much.
+     */
     {"JkLogFile", jk_set_log_file, NULL, RSRC_CONF, TAKE1,
      "Full path to the Jakarta Tomcat module log file"},
     {"JkLogLevel", jk_set_log_level, NULL, RSRC_CONF, TAKE1,
      "The Jakarta Tomcat module log level, can be debug, info, error or emerg"},
+
+    /*
+     * Apache has multiple SSL modules (for example apache_ssl, stronghold
+     * IHS ...). Each of these can have a different SSL environment names
+     * The following properties let the administrator specify the envoiroment
+     * variables names.
+     *
+     * HTTPS - indication for SSL 
+     * CERTS - Base64-Der-encoded client certificates.
+     * CIPHER - A string specifing the ciphers suite in use.
+     * SESSION - A string specifing the current SSL session.
+     */
+    {"JkHTTPSIndicator", jk_set_https_indicator, NULL, RSRC_CONF, TAKE1,
+     "Name of the Apache environment that contains SSL indication"},
+    {"JkCERTSIndicator", jk_set_certs_indicator, NULL, RSRC_CONF, TAKE1,
+     "Name of the Apache environment that contains SSL client certificates"},
+    {"JkCIPHERIndicator", jk_set_cipher_indicator, NULL, RSRC_CONF, TAKE1,
+     "Name of the Apache environment that contains SSL client cipher"},
+    {"JkSESSIONIndicator", jk_set_sesion_indicator, NULL, RSRC_CONF, TAKE1,
+     "Name of the Apache environment that contains SSL session"},
+    {"JkExtractSSL", jk_set_enable_ssl, NULL, RSRC_CONF, FLAG,
+     "Turns on SSL processing and information gathering by mod_jk"},     
+
+
+    {"JkEnvVar", jk_add_env_var, NULL, RSRC_CONF, TAKE2,
+     "Adds a name of environment variable that should be sent to Tomcat"},     
     {NULL}
 };
 
 /* ========================================================================= */
 /* The JK module handlers                                                    */
 /* ========================================================================= */
-
-ap_status_t jk_cleanup_endpoint( void *data ) {
-    jk_endpoint_t *end = (jk_endpoint_t *)data;    
-    /*     printf("XXX jk_cleanup1 %ld\n", data); */
-    end->done(&end, NULL);  
-    return 0;
-}
-
 static int jk_handler(request_rec *r)
 {   
     const char *worker_name = ap_table_get(r->notes, JK_WORKER_ID);
 
-    /* If this is a proxy request, we'll notify an error */
     if(r->proxyreq) {
         return HTTP_INTERNAL_SERVER_ERROR;
     }
@@ -481,39 +690,19 @@ static int jk_handler(request_rec *r)
             private_data.r = r;
 
             jk_init_ws_service(&s);
-
             s.ws_private = &private_data;
             s.pool = &private_data.p;            
             
-            if(init_ws_service(&private_data, &s)) {
+            if(init_ws_service(&private_data, &s, conf)) {
                 jk_endpoint_t *end = NULL;
-
-		/* Use per/thread pool ( or "context" ) to reuse the 
-		   endpoint. It's a bit faster, but I don't know 
-		   how to deal with load balancing - but it's usefull for JNI
-		*/
-
-#ifdef REUSE_WORKER
-		ap_pool_t *rpool=r->pool;
-		ap_pool_t *tpool=rpool->parent->parent;
-		
-		ap_get_userdata( &end, "jk_thread_endpoint", tpool );
-                if(end==NULL ) {
-		    worker->get_endpoint(worker, &end, l);
-		    ap_set_userdata( end , "jk_thread_endpoint", &jk_cleanup_endpoint,  tpool );
-		}
-#else
-		worker->get_endpoint(worker, &end, l);
-#endif
-		{   
-		    int is_recoverable_error = JK_FALSE;
+                if(worker->get_endpoint(worker, &end, l)) {
+                    int is_recoverable_error = JK_FALSE;
                     rc = end->service(end, 
                                       &s, 
                                       l, 
                                       &is_recoverable_error);
-#ifndef REUSE_WORKER		    
-		    end->done(&end, l); 
-#endif
+                
+                    end->done(&end, l);
                 }
             }
 
@@ -526,7 +715,7 @@ static int jk_handler(request_rec *r)
     return HTTP_INTERNAL_SERVER_ERROR;
 }
 
-static void *create_jk_config(ap_pool_t *p, server_rec *s)
+static void *create_jk_config(ap_pool *p, server_rec *s)
 {
     jk_server_conf_t *c =
         (jk_server_conf_t *) ap_pcalloc(p, sizeof(jk_server_conf_t));
@@ -536,25 +725,65 @@ static void *create_jk_config(ap_pool_t *p, server_rec *s)
     c->log_level   = -1;
     c->log         = NULL;
     c->mountcopy   = JK_FALSE;
-    c->was_initialized = JK_FALSE;
+
+    /*
+     * By default we will try to gather SSL info. 
+     * Disable this functionality through JkExtractSSL
+     */
+    c->ssl_enable  = JK_TRUE;
+    /*
+     * The defaults ssl indicators match those in mod_ssl (seems 
+     * to be in more use).
+     */
+    c->https_indicator  = "HTTPS";
+    c->certs_indicator  = "SSL_CLIENT_CERT";
+    
+    /*
+     * The following (comented out) environment variables match apache_ssl! 
+     * If you are using apache_sslapache_ssl uncomment them (or use the 
+     * configuration directives to set them.
+     *
+    c->cipher_indicator = "HTTPS_CIPHER";
+    c->sesion_indicator = NULL;
+     */
+
+    /*
+     * The following environment variables match mod_ssl! If you
+     * are using another module (say apache_ssl) comment them out.
+     */
+    c->cipher_indicator = "SSL_CIPHER";
+    c->sesion_indicator = "SSL_SESSION_ID";
 
     if(!map_alloc(&(c->uri_to_context))) {
-        jk_error_exit(APLOG_MARK, APLOG_EMERG, s, "Memory error");
+        jk_error_exit(APLOG_MARK, APLOG_EMERG, s, p, "Memory error");
     }
     c->uw_map = NULL;
+
+
+    c->envvars_in_use = JK_FALSE;
+    c->envvars = ap_make_table(p, 0);
+
     c->s = s;
 
     return c;
 }
 
 
-static void *merge_jk_config(ap_pool_t *p, 
+static void *merge_jk_config(ap_pool *p, 
                              void *basev, 
                              void *overridesv)
 {
     jk_server_conf_t *base = (jk_server_conf_t *) basev;
     jk_server_conf_t *overrides = (jk_server_conf_t *)overridesv;
- 
+
+    if(base->ssl_enable) {
+        overrides->ssl_enable       = base->ssl_enable;
+        overrides->https_indicator  = base->https_indicator;
+        overrides->certs_indicator  = base->certs_indicator;
+        overrides->cipher_indicator = base->cipher_indicator;
+        overrides->sesion_indicator = base->sesion_indicator;
+    }
+    
     if(overrides->mountcopy) {
         int sz = map_size(base->uri_to_context);
         int i;
@@ -566,37 +795,56 @@ static void *merge_jk_config(ap_pool_t *p,
                             name,
                             ap_pstrdup(p, map_get_string(base->uri_to_context, name, NULL)),
                             &old)) {
-                    jk_error_exit(APLOG_MARK, APLOG_EMERG, overrides->s, "Memory error");
+                    jk_error_exit(APLOG_MARK, 
+                                  APLOG_EMERG, 
+                                  overrides->s, 
+                                  p, 
+                                  "Memory error");
                 }
             }
         }
     }
+
+    if(base->envvars_in_use) {
+        overrides->envvars_in_use = JK_TRUE;
+        
+        overrides->envvars = 
+                ap_overlay_tables(p, 
+                                  overrides->envvars, 
+                                  base->envvars);
+
+    }
+
     if(overrides->log_file && overrides->log_level >= 0) {
-        if(!jk_open_file_logger(&(overrides->log), overrides->log_file, overrides->log_level)) {
+        if(!jk_open_file_logger(&(overrides->log), 
+                                overrides->log_file, 
+                                overrides->log_level)) {
             overrides->log = NULL;
         }
     }
     if(!uri_worker_map_alloc(&(overrides->uw_map), 
                              overrides->uri_to_context, 
                              overrides->log)) {
-        jk_error_exit(APLOG_MARK, APLOG_EMERG, overrides->s, "Memory error");
+        jk_error_exit(APLOG_MARK, 
+                      APLOG_EMERG, 
+                      overrides->s,  
+                      p, 
+                      "Memory error");
     }
     
     return overrides;
 }
 
-static void jk_child_init(ap_pool_t *pconf, 
-			  server_rec *s)
+static void jk_init(server_rec *s, ap_pool *p)
 {
-    char *p = getenv("WAS_BORN_BY_APACHE");
     jk_map_t *init_map = NULL;
     jk_server_conf_t *conf =
         (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
 
-    fprintf(stdout, "jk_child_init %s\n", p ? p : "NULL"); fflush(stdout);
-        
     if(conf->log_file && conf->log_level >= 0) {
-        if(!jk_open_file_logger(&(conf->log), conf->log_file, conf->log_level)) {
+        if(!jk_open_file_logger(&(conf->log), 
+                                conf->log_file, 
+                                conf->log_level)) {
             conf->log = NULL;
         } else {
             main_log = conf->log;
@@ -604,68 +852,33 @@ static void jk_child_init(ap_pool_t *pconf,
     }
     
     if(!uri_worker_map_alloc(&(conf->uw_map), conf->uri_to_context, conf->log)) {
-	printf( "Memory error - uri worker alloc \n");
-        jk_error_exit(APLOG_MARK, APLOG_EMERG, s, "Memory error");
+        jk_error_exit(APLOG_MARK, 
+                      APLOG_EMERG, 
+                      s, 
+                      p, 
+                      "Memory error");
     }
 
     if(map_alloc(&init_map)) {
         if(map_read_properties(init_map, conf->worker_file)) {
-	    if(wc_open(init_map, conf->log)) {
-		return;
-        }            
+
+#if MODULE_MAGIC_NUMBER >= 19980527
+    			/* Tell apache we're here */
+    			ap_add_version_component("mod_jk");
+#endif
+
+                if(wc_open(init_map, conf->log)) {
+		    map_free(&init_map);  /* we don't need this any more so free it */
+                    return;
+                }            
         }
     }
 
-    jk_error_exit(APLOG_MARK, APLOG_EMERG, s, "Error while opening the workers");
-}
-
-static void jk_post_config(ap_pool_t *pconf, 
-                           ap_pool_t *plog, 
-                           ap_pool_t *ptemp, 
-                           server_rec *s)
-{
-    if(!s->is_virtual) {
-        char *p = getenv("WAS_BORN_BY_APACHE");
-        jk_map_t *init_map = NULL;
-        jk_server_conf_t *conf =
-            (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
-        if(!conf->was_initialized) {
-            fprintf(stdout, "jk_post_config %s %s %d %d %s\n", 
-                    s->server_hostname, 
-                    s->server_admin,
-                    s,
-                    conf,
-                    p ? p : "NULL"); fflush(stdout);
-            
-            conf->was_initialized = JK_TRUE;        
-            if(conf->log_file && conf->log_level >= 0) {
-                if(!jk_open_file_logger(&(conf->log), conf->log_file, conf->log_level)) {
-                    conf->log = NULL;
-                } else {
-                    main_log = conf->log;
-                }
-            }
-    
-            if(!uri_worker_map_alloc(&(conf->uw_map), conf->uri_to_context, conf->log)) {
-                jk_error_exit(APLOG_MARK, APLOG_EMERG, s, "Memory error");
-            }
-
-            if(map_alloc(&init_map)) {
-                if(map_read_properties(init_map, conf->worker_file)) {
-                    if(!p) {
-                        putenv("WAS_BORN_BY_APACHE=true");
-                        return;
-                    } else {                        
-                        if(wc_open(init_map, conf->log)) {
-                            return;
-                        }            
-                    }
-                }
-            }
-
-            jk_error_exit(APLOG_MARK, APLOG_EMERG, s, "Error while opening the workers");
-        }
-    }
+    jk_error_exit(APLOG_MARK, 
+                  APLOG_EMERG, 
+                  s, 
+                  p, 
+                  "Error while opening the workers");
 }
 
 static int jk_translate(request_rec *r)
@@ -680,9 +893,8 @@ static int jk_translate(request_rec *r)
                                              conf->log ? conf->log : main_log);
 
             if(worker) {
-                r->handler=ap_pstrdup(r->pool,JK_HANDLER);
+                r->handler = ap_pstrdup(r->pool, JK_HANDLER);
                 ap_table_setn(r->notes, JK_WORKER_ID, worker);
-                return OK;
             }
         }
     }
@@ -690,24 +902,16 @@ static int jk_translate(request_rec *r)
     return DECLINED;
 }
 
-static void jk_register_hooks(void)
+static void exit_handler (server_rec *s, ap_pool *p)
 {
-#ifdef WIN32
-    ap_hook_post_config(jk_post_config,
-                        NULL,
-                        NULL,
-                        AP_HOOK_MIDDLE);    
+   jk_server_conf_t *conf =
+       (jk_server_conf_t *)ap_get_module_config(s->module_config, &jk_module);
 
-#else
-    ap_hook_child_init(jk_child_init,
-                       NULL,
-                       NULL,
-                       AP_HOOK_MIDDLE);    
-#endif
-    ap_hook_translate_name(jk_translate,
-                           NULL,
-                           NULL,
-                           AP_HOOK_FIRST);    
+   wc_close(conf->log);
+   uri_worker_map_free(&(conf->uw_map), conf->log);
+   map_free(&(conf->uri_to_context));
+   if (conf->log)
+      jk_close_file_logger(&(conf->log));
 }
 
 static const handler_rec jk_handlers[] =
@@ -717,14 +921,37 @@ static const handler_rec jk_handlers[] =
     { NULL }
 };
 
-module MODULE_VAR_EXPORT jk_module =
-{
-    STANDARD20_MODULE_STUFF,
-    NULL,	            /* dir config creater */
-    NULL,	            /* dir merger --- default is to override */
-    create_jk_config,	/* server config */
-    merge_jk_config,	/* merge server config */
-    jk_cmds,			/* command ap_table_t */
-    jk_handlers,		/* handlers */
-    jk_register_hooks	/* register hooks */
+module MODULE_VAR_EXPORT jk_module = {
+    STANDARD_MODULE_STUFF,
+    jk_init,                    /* module initializer */
+    NULL,                       /* per-directory config creator */
+    NULL,                       /* dir config merger */
+    create_jk_config,           /* server config creator */
+    merge_jk_config,            /* server config merger */
+    jk_cmds,                    /* command table */
+    jk_handlers,                /* [7] list of handlers */
+    jk_translate,               /* [2] filename-to-URI translation */
+    NULL,                       /* [5] check/validate user_id */
+    NULL,                       /* [6] check user_id is valid *here* */
+    NULL,                       /* [4] check access by host address */
+    NULL,                       /* XXX [7] MIME type checker/setter */
+    NULL,                       /* [8] fixups */
+    NULL,                       /* [10] logger */
+    NULL,                       /* [3] header parser */
+    NULL,                       /* apache child process initializer */
+    exit_handler,               /* apache child process exit/cleanup */
+    NULL                        /* [1] post read_request handling */
+#ifdef EAPI
+    /*
+     * Extended module APIs, needed when using SSL.
+     * STDC say that we do not have to have them as NULL but
+     * why take a chance
+     */
+    ,NULL,                      /* add_module */
+    NULL,                       /* remove_module */
+    NULL,                       /* rewrite_command */
+    NULL,                       /* new_connection */
+    NULL                       /* close_connection */
+#endif /* EAPI */
+
 };
