@@ -57,7 +57,7 @@
  * Description: ajpv1.2 worker, used to call local or remote jserv hosts   *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:    jserv_ajpv12.c from Jserv                                  *
- * Version:     $Revision: 1.2 $                                               *
+ * Version:     $Revision: 1.2.2.1 $                                               *
  ***************************************************************************/
 
 #include "jk_ajp12_worker.h"
@@ -465,6 +465,11 @@ static int ajpv12_handle_request(ajp12_endpoint_t *p,
                     return JK_FALSE;
                 }
                 jk_log(l, JK_LOG_DEBUG, "ajpv12_handle_request, sent %d bytes\n", this_time);
+            } else if (this_time == 0) {
+             jk_log(l, JK_LOG_ERROR,
+                     "In ajpv12_handle_request, Error: short read. content length is %d, read %d\n",
+                     s->content_length, so_far);
+             return JK_FALSE;
             }
         }
     }
