@@ -1,7 +1,7 @@
 /*
- * $Header: /home/cvspublic/jakarta-tomcat/src/share/org/apache/jasper/JspC.java,v 1.15.2.1 2000/07/12 16:04:20 shemnon Exp $
- * $Revision: 1.15.2.1 $
- * $Date: 2000/07/12 16:04:20 $
+ * $Header: /home/cvs/jakarta-tomcat/src/share/org/apache/jasper/JspC.java,v 1.15.2.3 2001/01/12 04:46:58 larryi Exp $
+ * $Revision: 1.15.2.3 $
+ * $Date: 2001/01/12 04:46:58 $
  *
  * ====================================================================
  * 
@@ -174,6 +174,11 @@ public class JspC implements Options { //, JspCompilationContext {
         return true;
     }
  
+    public boolean getClassDebugInfo() {
+        // compile with debug info
+        return false;
+    }
+
     public String getIeClassId() {
         return ieClassId;
     }
@@ -200,6 +205,10 @@ public class JspC implements Options { //, JspCompilationContext {
         return null;
     }
 
+    public String getJavaEncoding() {
+	return "UTF-8";
+    }
+
     public String getClassPath() {
         return System.getProperty("java.class.path");
     }
