@@ -282,17 +282,9 @@ public class JDBCRealm extends BaseInterceptor {
     public synchronized boolean authenticate(String username, String credentials) {
         try {
 
-            // Establish the database connection if necessary
-            if ((dbConnection == null) || dbConnection.isClosed()) {
-                log(sm.getString("jdbcRealm.authDBClosed"));
-                dbConnection = DriverManager.getConnection(connectionURL);
-                if( (dbConnection == null) || dbConnection.isClosed() ) {
-                    log(sm.getString("jdbcRealm.authDBReOpenFail"));
-                    return false;
-                }
-                dbConnection.setReadOnly(true);
+            if (!checkConnection()) {
+                return false;
             }
-
             // Create the authentication search prepared statement if necessary
             if (preparedAuthenticate == null) {
                 String sql = "SELECT " + userCredCol + " FROM " + userTable +
@@ -307,7 +299,7 @@ public class JDBCRealm extends BaseInterceptor {
             ResultSet rs1 = preparedAuthenticate.executeQuery();
             boolean found = false;
             if (rs1.next()) {
-                if (credentials.equals(rs1.getString(1))) {
+                if (credentials.equals(rs1.getString(1).trim())) {
                     if (debug >= 2)
                         log(sm.getString("jdbcRealm.authenticateSuccess",
                                  username));
@@ -352,15 +344,8 @@ public class JDBCRealm extends BaseInterceptor {
 
     public synchronized String[] getUserRoles(String username) {
         try {
-          if( (dbConnection == null) || dbConnection.isClosed() ) {
-            log(sm.getString("jdbcRealm.getUserRolesDBClosed"));
-
-            dbConnection = DriverManager.getConnection(connectionURL);
-
-            if( dbConnection == null || dbConnection.isClosed() ) {
-              log(sm.getString("jdbcRealm.getUserRolesDBReOpenFail"));
-              return null;
-            }
+          if( !checkConnection()) {
+                return null;
           }
           if (preparedRoles == null) {
                 String sql = "SELECT " + roleNameCol + " FROM " +
@@ -379,7 +364,7 @@ public class JDBCRealm extends BaseInterceptor {
           Vector vrol=new Vector();
 
           while (rs.next()) {
-              vrol.addElement(rs.getString(1));
+              vrol.addElement(rs.getString(1).trim());
           }
 
           String[] res=new String[vrol.size()];
@@ -419,25 +404,9 @@ public class JDBCRealm extends BaseInterceptor {
     public void contextInit(Context ctx)
             throws org.apache.tomcat.core.TomcatException {
 	// Validate and update our current component state
-      if (!started) {
+      if (!started && checkConnection() ) {
           started = true;
-          try {
-            Class.forName(driverName);
-            if ((connectionName == null || connectionName.equals("")) &&
-                (connectionPassword == null || connectionPassword.equals(""))) {
-                dbConnection = DriverManager.getConnection(connectionURL);
-            } else {
-                dbConnection = DriverManager.getConnection(connectionURL,
-                                                           connectionName,
-                                                           connectionPassword);
-            }
-          }
-          catch( ClassNotFoundException ex ) {
-            throw new RuntimeException("JDBCRealm.start.readXml: " + ex);
-          }
-          catch( SQLException ex ) {
-            throw new RuntimeException("JDBCRealm.start.readXml: " + ex);
-          }
+          log(sm.getString("jdbcRealm.started"));
       }
     }
 
@@ -445,6 +414,7 @@ public class JDBCRealm extends BaseInterceptor {
             throws org.apache.tomcat.core.TomcatException {
       // Validate and update our current component state
       if (started) {
+            started=false;
             if( dbConnection != null ) {
               try {
                 dbConnection.close();
@@ -479,12 +449,15 @@ public class JDBCRealm extends BaseInterceptor {
         String user=(String)cred.get("username");
         String password=(String)cred.get("password");
 	
-	if( authenticate( user, password ) ) {
-     	    if( debug > 0 ) log( "Auth ok, user=" + user );
-	    req.setRemoteUser( user );
-            Context ctx = req.getContext();
-            if (ctx != null)
-                req.setAuthType(ctx.getAuthMethod());
+	if( user !=null && password !=null ){
+            if ( authenticate( user, password ) ) {
+                if( debug > 0 ) log( "Auth ok, user=" + user );
+                req.setRemoteUser( user );
+                req.setUserPrincipal(new SimplePrincipal(user));
+                Context ctx = req.getContext();
+                if (ctx != null)
+                    req.setAuthType(ctx.getAuthMethod());
+            }
 	}
 	return 0;
     }
@@ -531,6 +504,36 @@ public class JDBCRealm extends BaseInterceptor {
         // XXX check transport
     }
 
+    private boolean checkConnection(){
+        if (started) log(sm.getString("jdbcRealm.checkConnectionDBClosed"));
+        else log(sm.getString("jdbcRealm.starting"));
+        try {
+            if( (dbConnection == null) || dbConnection.isClosed() ) {
+                Class.forName(driverName);
+                if ((connectionName == null || connectionName.equals("")) ||
+                        (connectionPassword == null || connectionPassword.equals(""))) {
+                        dbConnection = DriverManager.getConnection(connectionURL);
+                } else {
+                        dbConnection = DriverManager.getConnection(connectionURL,
+                                                                   connectionName,
+                                                                   connectionPassword);
+                }
+                if( dbConnection == null || dbConnection.isClosed() ) {
+                  log(sm.getString("jdbcRealm.checkConnectionDBReOpenFail"));
+                  return false;
+                }
+            }
+            return true;
+        }catch (SQLException ex){
+            log(sm.getString("jdbcRealm.checkConnectionSQLException"));
+            log ("SQLException: "+ex);
+            return false;
+        }
+        catch( ClassNotFoundException ex ) {
+            throw new RuntimeException("JDBCRealm.checkConnection: " + ex);
+        }
+    }
+
 }
 
 
